local _, AT = ...
local DEBUG = false
local function _assert(aCnd, aMsg) if DEBUG then assert(aCnd, aMsg) end return aCnd end

-- file import
local Constants = AT.Constants

local Utils = {}
AT.Utils = Utils

function Utils:Print(aMsg)
  print(string.format("|cff4fd0ff%s:|r %s", Constants.ADDON_DISPLAY_NAME, aMsg))
end

-- @RETURNS Number width
-- @RETURNS Number height
function Utils:GetResolution()
  local x, y = strsplit('x', Display_DisplayModeDropDown:windowedmode() and GetCVar('gxWindowedResolution') or GetCVar('gxFullscreenResolution'))
  return tonumber(x), tonumber(y)
end

-- @RETURNS Number pos x
-- @RETURNS Number pos y
-- @RETURNS Number map id
function Utils:GetUnitMapPosition(aUnit)
  local map = C_Map.GetBestMapForUnit(aUnit)
  local pos = C_Map.GetPlayerMapPosition(map, aUnit)
  local x = 0
  local y = 0
  if pos then
    x, y = pos:GetXY()
  end
  return x, y, map
end

-- @RETURNS String like WoW aura time
function Utils:FormatExpirationTime(aExpirationTime)
  if aExpirationTime <= 0 then
    return '0s';
  end

  if aExpirationTime < 60 then
    return aExpirationTime..'s'
  end

  if aExpirationTime < 3600 then
    return math.floor(aExpirationTime / 60)..'m'
  end

  return math.floor(aExpirationTime / 3600)..'h'
end

-- @RETURNS Table
function Utils:TableShallowCopy(aTable)
  local copy = {}
  for k, v in pairs(aTable) do
      copy[k] = v
  end
  return copy
end

-- @RETURNS Table
function Utils:TableDeepCopy(aTable)
  local copy = {}
  for k, v in pairs(aTable) do
      if type(orig) == 'table' then
        copy[k] = self:TableDeepCopy(v)
      else
        copy[k] = v
      end
  end
  return copy
end

-- @RETURNS Number
function Utils:TableLen(aTable)
  local len = 0
  for _, _ in pairs(aTable) do
    len = len + 1
  end
  return len
end

-- @RETURNS Number
function Utils:TableValueIndex(aTable, aVal)
  for i, v in ipairs(aTable) do
    if aVal == v then
      return i
    end
  end
end

-- @RETURNS Table number indexed
function Utils:TableKeys(aTable)
  local t = {}
  for k, _ in pairs(aTable) do
    tinsert(t, k)
  end
  return t
end

-- @RETURNS Boolean
function Utils:StrIsNilOrEmpty(aStr)
  return aStr == nil or aStr == ''
end

-- @RETURNS Boolean
function Utils:StrIsNilOrEmptyOrWhitespaceOnly(aStr)
  return Utils:StrIsNilOrEmpty(aStr) or strtrim(aStr) == ''
end

-- @RETURNS Boolean
function Utils:StrStartsWith(aStr1, aStr2)
  return aStr1:sub(1, aStr2:len()) == aStr2
end

--@ RETURNS String
function Utils:StrFirstCharToUpper(aStr)
  return aStr:gsub("^%l", strupper)
end

-- @RETURNS Number rounded
function Utils:Round(aNumber, aDecimals)
  local mult = 10^(aDecimals or 0)
  return math.floor(aNumber * mult + 0.5) / mult
end

-- @RETURNS Number clamped
function Utils:Clamp(aLow, aNumber, aHigh)
  return math.min(math.max(aNumber, aLow), aHigh)
end

-- @RETURNS aTrue or aFalse
function Utils:Ternary(aCond, aTrue, aFalse)
  if aCond then
    return aTrue
  end
  return aFalse
end

-- @RETURNS String hex color formatted for usage in escape sequences
function Utils:RgbToHexStr(aColor)
  local result = 'ff'
  local colors = { aColor.r, aColor.g, aColor.b }
  for _, v in ipairs(colors) do
    local str = string.format('%x', v * 255)
    if str:len() == 1 then
      str = '0'..str
    end
    _assert(str:len() == 2)
    result = result..str
  end
  return result
end
